@@ -103,6 +103,7 @@
 #' @importFrom future.apply future_lapply
 #' @examples
 #' \donttest{
+#' set.seed(1234)
 #' data("simPoisson")
 #' dat <- simPoisson[1:100,]
 #' mod1 <- spGLMstack(y ~ x1, data = dat, family = "poisson",
@@ -277,7 +278,7 @@ spGLMstack <- function(formula, data = parent.frame(), family,
         stop("priors[['nu.beta']] must be a single numeric value.")
       }
       if(nu.beta < 2.1){
-        warning("Supplied nu.beta is less than 2.1. Setting it to defaults.")
+        message("Supplied nu.beta is less than 2.1. Setting it to defaults.")
         nu.beta <- 2.1
       }
     }
@@ -290,7 +291,7 @@ spGLMstack <- function(formula, data = parent.frame(), family,
         stop("priors[['nu.z']] must be a single numeric value.")
       }
       if(nu.z < 2.1){
-        warning("Supplied nu.z is less than 2.1. Setting it to defaults.")
+        message("Supplied nu.z is less than 2.1. Setting it to defaults.")
         nu.z <- 2.1
       }
     }
@@ -307,7 +308,7 @@ spGLMstack <- function(formula, data = parent.frame(), family,
       }
     }
     if(missing.flag > 0){
-    warning("Some priors were not supplied. Using defaults.")
+    message("Some priors were not supplied. Using defaults.")
     }
   }
 
@@ -332,19 +333,19 @@ spGLMstack <- function(formula, data = parent.frame(), family,
 
     if(cor.fn == 'matern'){
       if(!"nu" %in% names(params.list)){
-        warning("warning: candidate values of nu not specified. Using defaults
+        message("Candidate values of nu not specified. Using defaults
                 c(0.5, 1, 1.5).")
         params.list[["nu"]] <- c(0.5, 1.0, 1.5)
       }
     }else{
       if("nu" %in% names(params.list)){
-        warning("cor.fn = 'exponential'. Ignoring candidate values of nu.")
+        message("cor.fn = 'exponential'. Ignoring candidate values of nu.")
       }
       params.list[["nu"]] <- c(0.0)
     }
 
     if(!"boundary" %in% names(params.list)){
-      warning("warning: candidate values of boundary not specified. Using
+      message("Candidate values of boundary not specified. Using
               defaults c(0.5, 0.75).")
       params.list[["boundary"]] <- c(0.5, 0.75)
     }
@@ -387,34 +388,34 @@ spGLMstack <- function(formula, data = parent.frame(), family,
     }
     if(loopd.method == "cv"){
       if(n < 100){
-        warning("Sample size too low for CV. Finding exact LOO-PD.")
+        message("Sample size too low for CV. Finding exact LOO-PD.")
         loopd.method <- "exact"
         CV.K <- as.integer(0)
       }else{
         if(!"cv.k" %in% names(loopd.controls)){
-          warning("warning: CV.K missing from loopd.controls. Using defaults.")
+          message("CV.K missing from loopd.controls. Using defaults.")
           CV.K <- 10
         }
         CV.K <- loopd.controls[["cv.k"]]
         if(CV.K < 10){
-          warning("CV.K must be at least 10. Setting it to 10.")
+          message("CV.K must be at least 10. Setting it to 10.")
           CV.K <- 10
         }else if(CV.K > 20){
-          warning("CV.K must be at most 20. Setting it to 20.")
+          message("CV.K must be at most 20. Setting it to 20.")
           CV.K <- 20
         }
         if(floor(CV.K) != CV.K){
-          warning("CV.K must be integer. Setting it to nearest integer.")
+          message("CV.K must be integer. Setting it to nearest integer.")
         }
       }
     }
     if(!"nmc" %in% names(loopd.controls)){
-      warning("warning: nMC missing from loopd.controls. Using defaults.")
+      message("nMC missing from loopd.controls. Using defaults.")
       loopd.nMC <- 500
     }
     loopd.nMC <- loopd.controls[["nmc"]]
     if(loopd.nMC < 500){
-      warning("Number of Monte Carlo samples too low. Using defaults.")
+      message("Number of Monte Carlo samples too low. Using defaults = 500.")
       loopd.nMC = 500
     }
   }
@@ -484,7 +485,7 @@ spGLMstack <- function(formula, data = parent.frame(), family,
     # Get current plan invoked by future::plan() by the user
     current_plan <- future::plan()
     if(!inherits(current_plan, "sequential")){
-      warning("Parallelization plan other than 'sequential' setup but parallel
+      message("Parallelization plan other than 'sequential' setup but parallel
       is set to FALSE. Ignoring parallelization plan.")
     }
 
@@ -545,6 +546,7 @@ spGLMstack <- function(formula, data = parent.frame(), family,
   out$cor.fn <- cor.fn
   out$priors <- list(mu.beta = rep(0, p), V.beta = V.beta, nu.beta = nu.beta,
                      nu.z = nu.z, sigmasq.xi = sigmaSq.xi)
+  out$n.samples <- n.samples
   out$samples <- samps
   out$loopd <- loopd_list
   out$loopd.method <- loopd.controls
